/**
 * Object Type Definitions
 */
/**
   * @typedef {Object} Avatar
   * @property {String} name The name of the user
   * @property {String} photoURL An HTTPS or UHRP URL to a photo of the user
   */

/**
   * @typedef {Object} GetTransactionsTx
   * @property {String} txid The transaction ID
   * @property {Number} amount The number of satoshis added or removed from Dojo by this transaction
   * @property {String} status The current state of the transaction. Common statuses are `completed` and `waitingForSenderToSend`.
   * @property {String} senderPaymail The Paymail handle of the person who sent the transaction
   * @property {String} recipientPaymail The Paymail handle of the person who received the transaction
   * @property {Boolean} isOutgoing Whether or not the transaction was created with `createTransaction`
   * @property {String} note The human-readable tag for the transaction, provided by the person who initiated it
   * @property {String} created_at The time the transaction was registered with the Dojo
   * @property {String} referenceNumber The Dojo reference number for the transaction
   * @property {Array<String>} labels A set of all the labels affixed to the transaction
   */

/**
   * @typedef {Object} GetTransactionsResult
   * @property {Number} totalTransactions The number of transactions in the complete set
   * @property {Array<GetTransactionsTx>} transactions The specific transactions from the set that were requested, based on `limit` and `offset`
   */

/**
   * @typedef {Object} GetTxWithOutputsOutput
   * @property {String} script The hex representing the locking script of the output
   * @property {Number} satoshis The number of satoshis to put in the output
   */

/**
   * @typedef {Object} GetTxWithOutputsResult
   * @property {String} rawTx The serialized, signed transaction that is ready for broadcast
   * @property {String} referenceNumber The reference number that should now be provided back to `processTransaction (or `updateTransactionStatus`)
   * @property {Object} inputs This is the fully-formed `inputs` field of this transaction, as per the SPV Envelope specification.
   * @property {Number} amount The amount of the transaction
   */

/**
   * @typedef {Object} GetPendingTransactionsTx
   * @property {Number} amount The number of satoshis added or removed from Dojo by this transaction
   * @property {String} senderPaymail The Paymail handle of the person who sent the transaction
   * @property {String} created_at The time the transaction was registered with the Dojo
   * @property {String} referenceNumber The Dojo reference number for the transaction
   */

/**
   * @typedef {Object} TxRedeemableOutput
   * @property {Number} index The index of the output to redeem in the transaction
   * @property {Number} unlockingScriptLength The byte length of the unlocking script you intend to use to unlock this output
   */

/**
   * @typedef {Object} TxOutput
   * @property {Number} satoshis The amount of satoshis that will be in the output
   * @property {String} script The hex string representing the output locking script
   */

/**
   * @typedef {Object} InputTemplate Describes an input to a transaction, and if it is a Dojo input, instructs you on how to sign it. In addition to the fields documented here, the other SPV envelope fields from the SPV envelope specification are also provided, so that a complete envelope can be created.
   * @property {String} providedBy Identifies the provider(s) of the output(s) to redeem trom the associated transaction. Can be "dojo", "you" (for externally-provided inputs), or "you-and-dojo"
   * @property {Array<Number>} outputsToRedeem A set of numbers indicating the index of each output that should be redeemed from this transaction
   * @property {String} rawTx As specified in the SPV envelope specification, the serialized transaction is provided
   * @property {Object} [instructions] When "providedBy" is "dojo" or "you-and-dojo", this is an object whose keys are numbers (from "outputsToRedeem") that correspond to Dojo-provided inputs
   * @property {Object} [instructions[outputIndex]] This is an object that gives you instructions on how to sign a Dojo-provided input. It contains the type (P2PKH or P2RPH) and derivation path (from your root XPRIV key) of the key that unlocks the input
   * @property {String} [instructions[outputIndex].type] The type of the unlocking script you will need to produce with the key given by the derivation path
   * @property {String} [instructions[outputIndex].derivationPath] The derivation path (from your root XPRIV key) that will give you the key that you can use to create the unlocking script
   */

/**
   * @typedef {Object} OutputTemplate
   * @property {Number} satoshis The number of satoshis that will be put into this output
   * @property {String} script The hex string representing the output script
   * @property {String} providedBy Who is responsible for this output - Either "you" or "dojo"
   * @property {String} [purpose] When the output is provided by Dojo, its purpose can either be "change" or "service-charge"
   * @property {String} [destinationBasket] When the output is provided by Dojo, this indicates which basket it is destined for
   */

/**
   * @typedef {Object} TransactionTemplate A set of instructions that Dojo returns, instructing you on how to create and sign a transaction using keys you control (usually derived child keys from your Dojo XPRIV key).
   * @property {String} referenceNumber The reference number you will use to either process the transaction (with `processTransaction`) or cancel it (with `updateTransactionStatus`).
   * @property {Object<InputTemplate>} inputs The set of input templates for this transaction. This is an object whose keys are TXIDs and whose values are InputTemplate objects.
   * @property {Array<OutputTemplate>} outputs The set of output templates to include in this transaction
   */

/**
   * @typedef {Object} TxInputEnvelope An SPV envelope object for the associated
   *                 TXID. See the SPV envelopes specification for details, or
   *                 use [hashwrap](https://www.npmjs.com/package/hash-wrap) to
   *                 generate an envelope with the TXID. In addition to the fields
   *                 documented here, other fields from the Envelope
   *                 specification are required.
   * @property {String} rawTx The serialized hex of the transaction
   * @property {Object} [inputs] As specified by SPV Envelope
   * @property {Array<Object>} [mapiResponses] As specified by SPV Envelope
   * @property {Object} [proof] As specified by SPV Envelope
   * @property {Array<TxRedeemableOutput>} outputsToRedeem
   *                 In addition to the fields for a normal envelope, this is
   *                 used to indicate the outputs in the transaction that you are
   *                 redeeming here.
   */

/**
   * @typedef {Object} TransactionOutputDescriptor
   * @property {String} txid Transaction ID of transaction that created the output
   * @property {Number} vout Index in the transaction of the output
   * @property {Number} amount Number of satoshis in the output
   * @property {String} outputScript Hex representation of output locking script
   * @property {String} type The type of output, for example "P2PKH" or "P2RPH"
   * @property {Boolean} spendable Whether this output is free to be spent
   */

module.exports = {}
